/*
 * COVID-19 Bar chart race on LightningChartJS.
 */
import covid from './covid.json';
// Import LightningChartJS
const lcjs = require('@arction/lcjs')

// Extract required parts from LightningChartJS.
const {
    lightningChart,
    SolidFill,
    ColorHEX,
    AnimationEasings,
    Animator,
    UIOrigins,
    UILayoutBuilders,
    AxisTickStrategies,
    UIElementBuilders,
    AxisScrollStrategies,
    emptyLine,
    emptyFill,
    Themes
} = lcjs

const ls = lightningChart()

// Variables used in the example
const rectThickness = 1
const rectGap = 0.5
const bars = []
const duration = 2000 // duration of timer and animation

let y = 0
let initday = 15
// 2 days ago. the final day of the race
let yesterday = new Date(new Date().getFullYear(),new Date().getMonth() , new Date().getDate() - 2).toISOString() // ((d) => { d.setDate(d.getDate() - 2); return d })(new Date)
let connectionError = '' 

const barChart = options => {
    // Create a XY chart and add a RectSeries to it for rendering rectangles.
    const chart = ls.ChartXY(options)
        // Use Chart's title to track date
        .setTitle('COVID-19 cases ' + new Date(2020, 2, 15).toLocaleDateString(undefined, { year: "numeric", month: "long", day: "numeric" }))
        // Disable AutoCursor (hide information of bar by hover the mouse over it)
        .setAutoCursorMode(0)
        // Add padding to Chart's right side
        .setPadding({ right: 40 })
        .setMouseInteractions(false);

    // Cache X axis
    const axisX = chart.getDefaultAxisX()
        // Disable the scrolling animation for the X Axis, so it doesn't interfere.
        .setAnimationScroll(false)
        .setMouseInteractions(false);

    // Cache Y axis 
    const axisY = chart.getDefaultAxisY()
        // Hide default ticks
        .setTickStrategy(AxisTickStrategies.Empty)
        // Disable Mouse interactions for the Y Axis
        .setMouseInteractions(false)

    // Set custom label for Y-axis
    const createTickLabel = (entry, y) => {
        return axisY.addCustomTick(UIElementBuilders.AxisTick)
            .setValue(y + rectGap)
            .setGridStrokeLength(0)
            .setTextFormatter(_ => entry.country)
            .setMarker((marker) => marker
                .setTextFillStyle(new SolidFill({ color: ColorHEX('#aaaf') }))
                .setTextFont(fontSettings => fontSettings.setSize(17))
            )
    }

    // Function returns single bar with property of dimensions, data(entry) and label
    const addCountryHandler = entry => {
        const rectDimensions = {
            x: 0,
            y: y,
            width: entry.value,
            height: rectThickness
        }

        // Each country has its own rectangle series for different style.
        const rectSeries = chart.addRectangleSeries()
        const rect = rectSeries.add(rectDimensions)

        // Add TextBox element to the bar
        const label = chart.addUIElement(
            UILayoutBuilders.TextBox,
            { x: axisX, y: axisY }
        )
            .setOrigin(UIOrigins.LeftBottom)
            .setPosition({
                x: entry.value,
                y: y
            })
            .setText(entry.value.toLocaleString())
            .setTextFont(fontSettings => fontSettings.setSize(15))
            .setPadding(10)
            .setBackground((background) => background
                .setFillStyle(emptyFill)
                .setStrokeStyle(emptyLine)
            )

        // Set label title and position
        const tick = createTickLabel(entry, y)

        // Set interval for Y axis
        axisY.setInterval(-rectThickness, y)

        // Increase value of Y variable 
        y += rectThickness

        // Return figure
        return {
            entry,
            rect,
            label,
            tick,
        }
    }

    // Loop for adding bars
    const addCountries = (entries) => {
        for (const entry of entries) {
            bars.push(addCountryHandler(entry))
        }
    }

    // Sorting and splicing array of data
    const sortCountries = (data, raceDay) => {

        let myday = (raceDay.getMonth() + 1) + '/' + raceDay.getDate() + '/' + raceDay.getFullYear().toString().substr(-2)
        const countries = {...data}

        
        // Map list of countries and sort them in the order (First sort by value, then by country)
        const countryList = Object.values(countries).map((c: any) => (
            { country: c.country, value: c.history[myday] }
        )).sort((a, b) => (a.value > b.value) ? 1 : (a.value === b.value) ? ((a.country > b.country) ? 1 : -1) : -1)

        // Keep only top 20 countries
        countryList.splice(0, countryList.length - 20)

        return countryList
    }

    // Loop for re-rendering list of data
    const startRace = (data) => {

        // Inital day of race
        let raceDay = new Date(2020, 2, initday)

        // Set title of chart 
        chart.setTitle(connectionError + ' COVID-19 cases ' + raceDay.toLocaleDateString(undefined, { year: "numeric", month: "long", day: "numeric" }))

        // Get sorting data
        const sortedCountries = sortCountries(data, raceDay)

        // Check if the functions(startRace) has already done first pass
        if (bars.length > 0) {
            for (let i = 0; i < sortedCountries.length; i++) {

                // Prevent automatic scrolling of Y axis
                axisY.setScrollStrategy(AxisScrollStrategies.progressive)

                // Get index of each bar before sorting
                const initY = bars.map((e) => (e.entry.country)).indexOf(sortedCountries[i].country);

                // Get index of each bar after sorting
                const finalY = sortedCountries.map((e) => (e.country)).indexOf(sortedCountries[i].country);

                // Get Dimensions, Position And Size of bar[i]
                const rectDimensions = bars[i].rect.getDimensionsPositionAndSize()

                // Animation of changing the position of bar[i] by Y axis
                bars[i].animator = Animator(() => { undefined }) // function that executes after animation 
                    // Time for animation and easing type
                    (duration, AnimationEasings.linear)
                    // functions gets 2 arrays of 2 values (range) - initY (prev Y pos) and finalY.y(new Y pos) and rectDimensions.width, sortedCountries[i].value - prev and next width of each bar 
                    // and creates loop with increasing intermediate value (newPos, newWidth)
                    ([[initY, finalY], [rectDimensions.width, sortedCountries[i].value]], ([animatedYPosition, animatedValue]) => {
                        // Reset values 
                        bars[i].entry.country = sortedCountries[i].country
                        // Animate x and y positions
                        bars[i].rect.setDimensions({
                            x: 0,
                            y: animatedYPosition,
                            width: animatedValue,
                            height: 0.98
                        })
                            .restore()

                        // Animate labels
                        bars[i].label
                            // The topmost country's label should be inside the bar, the others should be to the right of the bar.
                            .setOrigin(i !== bars.length - 1 ? UIOrigins.LeftCenter : UIOrigins.RightCenter)
                            // Position the label
                            .setPosition({
                                x: animatedValue,
                                y: animatedYPosition > 0 ? (animatedYPosition + rectGap) : rectGap
                            })
                            .setText(Math.round(animatedValue).toLocaleString())
                            .setPadding(10, 0, 10, 0)
                            // Disable mouse interactions for the label
                            .setMouseInteractions(false)
                        // update tick position
                        bars[i].tick.setValue(animatedYPosition + rectGap)
                    })
            }

        } else {
            // If function executes for the first time, add countries to the Chart.
            addCountries(sortedCountries)
        }

        // we've bot reached the last day (i.e. yesterday) execute the function again and increase the day
        if (raceDay.toISOString() !== yesterday) {
            initday++
            setTimeout(() => startRace(data), duration)
        }
    }

    // function is used for showing errors on line 328
    const setTitle = (msg) => {
        chart.setTitle(msg)
    }

    return {
        addCountries,
        startRace,
        setTitle
    }

}

const startRaceHandler = (chart) => {

    // Fetch all countries and history of cases
    fetch('https://www.arction.com/lightningchart-js-interactive-examples/data/covid/confirmed.json')
        .then(res => res.json())
        .then(data => {
            //Uncomment if want to use data from Lighning Charts
            
                const dat = [...data.locations]
                chart.startRace(mergeData(dat))
            

            //Uncomment if want to use json file
            //chart.startRace(mergeData(covid.members))
            
        })
        .catch(err => {
            yesterday = '2020-05-30T21:00:00.000Z'
            connectionError = 'Example of data'       
            chart.startRace(mergeData(covid.members))
        })

    // Some countries are divided by region and it is needed to megre them
    const mergeData = data => {
        const result = []
        // Return only specific information 
        data.forEach(basket => {

            let ob = {
                country: basket['country'],
                history: { ...basket['history'] }
            }

            // Check if array already contains the country
            let has = false
            result.find(obj => obj.country == ob.country
                ? has = true
                : has = false
            )

            // Unite object values
            const sum = (obj, newObj) => {
                Object.keys(obj).map(date => (
                    obj[date] += newObj[date]
                ))
            }

            // Format the data
            if (has) {
                result.find(obj => (
                    obj.country == ob.country
                    && sum(obj.history, ob.history))
                )
            } else {
                result.push(ob)
            }
        });

        return result;
    };
}

// Create chart
const newchart = barChart({
    theme: Themes.auroraBorealis
})

// Start fetching
startRaceHandler(newchart)


